# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from tests import IntegrationTestCase
from tests.holodeck import Request
from twilio.base.exceptions import TwilioException
from twilio.http.response import Response


class ConferenceTestCase(IntegrationTestCase):

    def test_fetch_request(self):
        self.holodeck.mock(Response(500, ''))

        with self.assertRaises(TwilioException):
            self.client.api.v2010.accounts("ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                 .conferences("CFXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX").fetch()

        self.holodeck.assert_has_request(Request(
            'get',
            'https://api.twilio.com/2010-04-01/Accounts/ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX/Conferences/CFXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX.json',
        ))

    def test_fetch_valid_mixer_zone_response(self):
        self.holodeck.mock(Response(
            200,
            '''
            {
                "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "api_version": "2010-04-01",
                "date_created": "Fri, 18 Feb 2011 19:26:50 +0000",
                "date_updated": "Fri, 18 Feb 2011 19:27:33 +0000",
                "friendly_name": "AHH YEAH",
                "sid": "CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "region": "us1",
                "status": "completed",
                "subresource_uris": {
                    "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Participants.json",
                    "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Recordings.json"
                },
                "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa.json",
                "reason_conference_ended": "last-participant-left",
                "call_sid_ending_conference": "CAaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
            }
            '''
        ))

        actual = self.client.api.v2010.accounts("ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                      .conferences("CFXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX").fetch()

        self.assertIsNotNone(actual)

    def test_fetch_valid_region_in_progress_response(self):
        self.holodeck.mock(Response(
            200,
            '''
            {
                "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "api_version": "2010-04-01",
                "date_created": "Fri, 18 Feb 2011 19:26:50 +0000",
                "date_updated": "Fri, 18 Feb 2011 19:27:33 +0000",
                "friendly_name": "AHH YEAH",
                "sid": "CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "region": "au1",
                "status": "in-progress",
                "subresource_uris": {
                    "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Participants.json",
                    "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Recordings.json"
                },
                "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa.json",
                "reason_conference_ended": null,
                "call_sid_ending_conference": null
            }
            '''
        ))

        actual = self.client.api.v2010.accounts("ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                      .conferences("CFXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX").fetch()

        self.assertIsNotNone(actual)

    def test_fetch_without_mixer_zone_integer_status_response(self):
        self.holodeck.mock(Response(
            200,
            '''
            {
                "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "api_version": "2010-04-01",
                "date_created": "Fri, 18 Feb 2011 19:26:50 +0000",
                "date_updated": "Fri, 18 Feb 2011 19:27:33 +0000",
                "friendly_name": "AHH YEAH",
                "sid": "CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "region": "us1",
                "status": "completed",
                "subresource_uris": {
                    "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Participants.json",
                    "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Recordings.json"
                },
                "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa.json",
                "reason_conference_ended": "participant-with-end-conference-on-exit-left",
                "call_sid_ending_conference": "CAaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
            }
            '''
        ))

        actual = self.client.api.v2010.accounts("ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                      .conferences("CFXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX").fetch()

        self.assertIsNotNone(actual)

    def test_fetch_unknown_mixer_zone_init_integer_status_response(self):
        self.holodeck.mock(Response(
            200,
            '''
            {
                "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "api_version": "2010-04-01",
                "date_created": "Fri, 18 Feb 2011 19:26:50 +0000",
                "date_updated": "Fri, 18 Feb 2011 19:27:33 +0000",
                "friendly_name": "AHH YEAH",
                "sid": "CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "region": "unknown",
                "status": "init",
                "subresource_uris": {
                    "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Participants.json",
                    "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Recordings.json"
                },
                "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa.json",
                "reason_conference_ended": "participant-with-end-conference-on-exit-left",
                "call_sid_ending_conference": "CAaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
            }
            '''
        ))

        actual = self.client.api.v2010.accounts("ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                      .conferences("CFXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX").fetch()

        self.assertIsNotNone(actual)

    def test_list_request(self):
        self.holodeck.mock(Response(500, ''))

        with self.assertRaises(TwilioException):
            self.client.api.v2010.accounts("ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                 .conferences.list()

        self.holodeck.assert_has_request(Request(
            'get',
            'https://api.twilio.com/2010-04-01/Accounts/ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX/Conferences.json',
        ))

    def test_read_empty_response(self):
        self.holodeck.mock(Response(
            200,
            '''
            {
                "conferences": [],
                "first_page_uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences.json?Status=init&DateUpdated%3E=2018-11-12&DateUpdated%3C=2018-11-11&DateCreated=2008-01-03&FriendlyName=friendly_name&DateUpdated=2018-11-13&DateCreated%3C=2008-01-01&DateCreated%3E=2008-01-02&PageSize=50&Page=0",
                "next_page_uri": null,
                "previous_page_uri": null,
                "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences.json?Status=init&DateUpdated%3E=2018-11-12&DateUpdated%3C=2018-11-11&DateCreated=2008-01-03&FriendlyName=friendly_name&DateUpdated=2018-11-13&DateCreated%3C=2008-01-01&DateCreated%3E=2008-01-02&PageSize=50&Page=0",
                "page": 0,
                "page_size": 50,
                "start": 0,
                "end": 0
            }
            '''
        ))

        actual = self.client.api.v2010.accounts("ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                      .conferences.list()

        self.assertIsNotNone(actual)

    def test_read_next_response(self):
        self.holodeck.mock(Response(
            200,
            '''
            {
                "conferences": [
                    {
                        "status": "in-progress",
                        "region": "jp1",
                        "sid": "CFdddddddddddddddddddddddddddddddd",
                        "date_updated": "Thu, 01 Jan 2015 10:23:45 +0000",
                        "date_created": "Thu, 01 Jan 2015 10:23:45 +0000",
                        "subresource_uris": {
                            "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFdddddddddddddddddddddddddddddddd/Participants.json",
                            "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFdddddddddddddddddddddddddddddddd/Recordings.json"
                        },
                        "friendly_name": "friendly_name",
                        "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFdddddddddddddddddddddddddddddddd.json",
                        "api_version": "2010-04-01",
                        "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "reason_conference_ended": null,
                        "call_sid_ending_conference": null
                    },
                    {
                        "status": "in-progress",
                        "region": "unknown",
                        "sid": "CFeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee",
                        "date_updated": "Thu, 01 Jan 2015 09:23:45 +0000",
                        "date_created": "Thu, 01 Jan 2015 09:23:45 +0000",
                        "subresource_uris": {
                            "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee/Participants.json",
                            "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee/Recordings.json"
                        },
                        "friendly_name": "friendly_name",
                        "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee.json",
                        "api_version": "2010-04-01",
                        "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "reason_conference_ended": null,
                        "call_sid_ending_conference": null
                    },
                    {
                        "status": "in-progress",
                        "region": "us1",
                        "sid": "CFffffffffffffffffffffffffffffffff",
                        "date_updated": "Thu, 01 Jan 2015 08:23:45 +0000",
                        "date_created": "Thu, 01 Jan 2015 08:23:45 +0000",
                        "subresource_uris": {
                            "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFffffffffffffffffffffffffffffffff/Participants.json",
                            "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFffffffffffffffffffffffffffffffff/Recordings.json"
                        },
                        "friendly_name": "friendly_name",
                        "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFffffffffffffffffffffffffffffffff.json",
                        "api_version": "2010-04-01",
                        "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "reason_conference_ended": null,
                        "call_sid_ending_conference": null
                    }
                ],
                "first_page_uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences.json?Status=in-progress&DateUpdated%3E=2018-11-12&DateUpdated%3C=2018-11-11&DateCreated=2008-01-03&FriendlyName=friendly_name&DateUpdated=2018-11-13&DateCreated%3C=2008-01-01&DateCreated%3E=2008-01-02&PageSize=3&Page=0",
                "next_page_uri": null,
                "previous_page_uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences.json?Status=in-progress&DateUpdated%3E=2018-11-12&DateUpdated%3C=2018-11-11&DateCreated=2008-01-03&FriendlyName=friendly_name&DateUpdated=2018-11-13&DateCreated%3C=2008-01-01&DateCreated%3E=2008-01-02&PageSize=3&Page=0&PageToken=PBCFdddddddddddddddddddddddddddddddd",
                "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences.json?Status=in-progress&DateUpdated%3E=2018-11-12&DateUpdated%3C=2018-11-11&DateCreated=2008-01-03&FriendlyName=friendly_name&DateUpdated=2018-11-13&DateCreated%3C=2008-01-01&DateCreated%3E=2008-01-02&PageSize=3&Page=1&PageToken=PACFcccccccccccccccccccccccccccccccc",
                "page": 1,
                "page_size": 3,
                "start": 3,
                "end": 5
            }
            '''
        ))

        actual = self.client.api.v2010.accounts("ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                      .conferences.list()

        self.assertIsNotNone(actual)

    def test_read_previous_response(self):
        self.holodeck.mock(Response(
            200,
            '''
            {
                "conferences": [
                    {
                        "status": "in-progress",
                        "region": "jp1",
                        "sid": "CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "date_updated": "Sat, 03 Jan 2015 11:23:45 +0000",
                        "date_created": "Sat, 03 Jan 2015 11:23:45 +0000",
                        "subresource_uris": {
                            "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Participants.json",
                            "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Recordings.json"
                        },
                        "friendly_name": "friendly_name",
                        "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa.json",
                        "api_version": "2010-04-01",
                        "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "reason_conference_ended": null,
                        "call_sid_ending_conference": null
                    },
                    {
                        "status": "in-progress",
                        "region": "unknown",
                        "sid": "CFbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb",
                        "date_updated": "Fri, 02 Jan 2015 11:23:45 +0000",
                        "date_created": "Fri, 02 Jan 2015 11:23:45 +0000",
                        "subresource_uris": {
                            "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb/Participants.json",
                            "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb/Recordings.json"
                        },
                        "friendly_name": "friendly_name",
                        "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb.json",
                        "api_version": "2010-04-01",
                        "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "reason_conference_ended": null,
                        "call_sid_ending_conference": null
                    },
                    {
                        "status": "in-progress",
                        "region": "us1",
                        "sid": "CFcccccccccccccccccccccccccccccccc",
                        "date_updated": "Thu, 01 Jan 2015 11:23:45 +0000",
                        "date_created": "Thu, 01 Jan 2015 11:23:45 +0000",
                        "subresource_uris": {
                            "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFcccccccccccccccccccccccccccccccc/Participants.json",
                            "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFcccccccccccccccccccccccccccccccc/Recordings.json"
                        },
                        "friendly_name": "friendly_name",
                        "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFcccccccccccccccccccccccccccccccc.json",
                        "api_version": "2010-04-01",
                        "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "reason_conference_ended": null,
                        "call_sid_ending_conference": null
                    }
                ],
                "first_page_uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences.json?Status=in-progress&DateUpdated%3E=2018-11-12&DateUpdated%3C=2018-11-11&DateCreated=2008-01-03&FriendlyName=friendly_name&DateUpdated=2018-11-13&DateCreated%3C=2008-01-01&DateCreated%3E=2008-01-02&PageSize=3&Page=0",
                "next_page_uri": null,
                "previous_page_uri": null,
                "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences.json?Status=in-progress&DateUpdated%3E=2018-11-12&DateUpdated%3C=2018-11-11&DateCreated=2008-01-03&FriendlyName=friendly_name&DateUpdated=2018-11-13&DateCreated%3C=2008-01-01&DateCreated%3E=2008-01-02&PageSize=3&Page=0&PageToken=PBCFdddddddddddddddddddddddddddddddd",
                "page": 0,
                "page_size": 3,
                "start": 0,
                "end": 2
            }
            '''
        ))

        actual = self.client.api.v2010.accounts("ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                      .conferences.list()

        self.assertIsNotNone(actual)

    def test_read_myroom_response(self):
        self.holodeck.mock(Response(
            200,
            '''
            {
                "conferences": [
                    {
                        "status": "in-progress",
                        "region": "jp1",
                        "sid": "CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "date_updated": "Sun, 03 Jan 2021 11:23:45 +0000",
                        "date_created": "Sun, 03 Jan 2021 11:23:45 +0000",
                        "subresource_uris": {
                            "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Participants.json",
                            "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Recordings.json"
                        },
                        "friendly_name": "MyRoom",
                        "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa.json",
                        "api_version": "2010-04-01",
                        "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "reason_conference_ended": null,
                        "call_sid_ending_conference": null
                    },
                    {
                        "status": "completed",
                        "region": "us1",
                        "sid": "CFbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb",
                        "date_updated": "Sat, 02 Jan 2021 11:23:45 +0000",
                        "date_created": "Sat, 02 Jan 2021 11:23:45 +0000",
                        "subresource_uris": {
                            "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb/Participants.json",
                            "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb/Recordings.json"
                        },
                        "friendly_name": "MyRoom",
                        "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb.json",
                        "api_version": "2010-04-01",
                        "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "reason_conference_ended": "last-participant-left",
                        "call_sid_ending_conference": "CAbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb"
                    },
                    {
                        "status": "completed",
                        "region": "ie1",
                        "sid": "CFcccccccccccccccccccccccccccccccc",
                        "date_updated": "Fri, 01 Jan 2021 11:23:45 +0000",
                        "date_created": "Fri, 01 Jan 2021 11:23:45 +0000",
                        "subresource_uris": {
                            "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFcccccccccccccccccccccccccccccccc/Participants.json",
                            "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFcccccccccccccccccccccccccccccccc/Recordings.json"
                        },
                        "friendly_name": "MyRoom",
                        "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFcccccccccccccccccccccccccccccccc.json",
                        "api_version": "2010-04-01",
                        "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "reason_conference_ended": "last-participant-left",
                        "call_sid_ending_conference": "CAcccccccccccccccccccccccccccccccc"
                    }
                ],
                "first_page_uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences.json?FriendlyName=MyRoom&PageSize=20&Page=0",
                "next_page_uri": null,
                "previous_page_uri": null,
                "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences.json?FriendlyName=MyRoom&PageSize=20&Page=0",
                "page": 0,
                "page_size": 20,
                "start": 0,
                "end": 2
            }
            '''
        ))

        actual = self.client.api.v2010.accounts("ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                      .conferences.list()

        self.assertIsNotNone(actual)

    def test_read_full_response(self):
        self.holodeck.mock(Response(
            200,
            '''
            {
                "conferences": [
                    {
                        "status": "in-progress",
                        "region": "jp1",
                        "sid": "CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "date_updated": "Fri, 03 Jul 2020 11:23:45 +0000",
                        "date_created": "Fri, 03 Jul 2020 11:23:45 +0000",
                        "subresource_uris": {
                            "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Participants.json",
                            "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Recordings.json"
                        },
                        "friendly_name": "friendly_name",
                        "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa.json",
                        "api_version": "2010-04-01",
                        "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "reason_conference_ended": null,
                        "call_sid_ending_conference": null
                    },
                    {
                        "status": "in-progress",
                        "region": "de1",
                        "sid": "CFbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb",
                        "date_updated": "Thu, 02 Jul 2020 11:23:45 +0000",
                        "date_created": "Thu, 02 Jul 2020 11:23:45 +0000",
                        "subresource_uris": {
                            "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb/Participants.json",
                            "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb/Recordings.json"
                        },
                        "friendly_name": "MyRoom",
                        "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb.json",
                        "api_version": "2010-04-01",
                        "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "reason_conference_ended": null,
                        "call_sid_ending_conference": null
                    },
                    {
                        "status": "completed",
                        "region": "br1",
                        "sid": "CFcccccccccccccccccccccccccccccccc",
                        "date_updated": "Wed, 01 Jul 2020 11:23:45 +0000",
                        "date_created": "Wed, 01 Jul 2020 11:23:45 +0000",
                        "subresource_uris": {
                            "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFcccccccccccccccccccccccccccccccc/Participants.json",
                            "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFcccccccccccccccccccccccccccccccc/Recordings.json"
                        },
                        "friendly_name": "FRIEND",
                        "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFcccccccccccccccccccccccccccccccc.json",
                        "api_version": "2010-04-01",
                        "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "reason_conference_ended": "participant-with-end-conference-on-exit-left",
                        "call_sid_ending_conference": "CAaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
                    }
                ],
                "first_page_uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences.json?PageSize=3&Page=0",
                "next_page_uri": null,
                "previous_page_uri": null,
                "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences.json?PageSize=3&Page=0",
                "page": 0,
                "page_size": 3,
                "start": 0,
                "end": 2
            }
            '''
        ))

        actual = self.client.api.v2010.accounts("ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                      .conferences.list()

        self.assertIsNotNone(actual)

    def test_read_date_created_equals_response(self):
        self.holodeck.mock(Response(
            200,
            '''
            {
                "conferences": [
                    {
                        "status": "in-progress",
                        "region": "jp1",
                        "sid": "CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "date_updated": "Tue, 07 Jul 2020 11:23:45 +0000",
                        "date_created": "Tue, 07 Jul 2020 11:23:45 +0000",
                        "subresource_uris": {
                            "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Participants.json",
                            "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Recordings.json"
                        },
                        "friendly_name": "friendly_name",
                        "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa.json",
                        "api_version": "2010-04-01",
                        "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "reason_conference_ended": null,
                        "call_sid_ending_conference": null
                    },
                    {
                        "status": "in-progress",
                        "region": "de1",
                        "sid": "CFbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb",
                        "date_updated": "Tue, 07 Jul 2020 11:23:45 +0000",
                        "date_created": "Tue, 07 Jul 2020 11:23:45 +0000",
                        "subresource_uris": {
                            "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb/Participants.json",
                            "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb/Recordings.json"
                        },
                        "friendly_name": "MyRoom",
                        "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb.json",
                        "api_version": "2010-04-01",
                        "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "reason_conference_ended": null,
                        "call_sid_ending_conference": null
                    },
                    {
                        "status": "completed",
                        "region": "br1",
                        "sid": "CFcccccccccccccccccccccccccccccccc",
                        "date_updated": "Tue, 07 Jul 2020 11:23:45 +0000",
                        "date_created": "Tue, 07 Jul 2020 11:23:45 +0000",
                        "subresource_uris": {
                            "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFcccccccccccccccccccccccccccccccc/Participants.json",
                            "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFcccccccccccccccccccccccccccccccc/Recordings.json"
                        },
                        "friendly_name": "FRIEND",
                        "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFcccccccccccccccccccccccccccccccc.json",
                        "api_version": "2010-04-01",
                        "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "reason_conference_ended": "participant-with-end-conference-on-exit-left",
                        "call_sid_ending_conference": "CAaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
                    }
                ],
                "first_page_uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences.json?DateCreated=2020-07-07&PageSize=3&Page=0",
                "next_page_uri": null,
                "previous_page_uri": null,
                "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences.json?DateCreated=2020-07-07&PageSize=3&Page=0",
                "page": 0,
                "page_size": 3,
                "start": 0,
                "end": 2
            }
            '''
        ))

        actual = self.client.api.v2010.accounts("ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                      .conferences.list()

        self.assertIsNotNone(actual)

    def test_read_date_created_on_or_after_response(self):
        self.holodeck.mock(Response(
            200,
            '''
            {
                "conferences": [
                    {
                        "status": "in-progress",
                        "region": "jp1",
                        "sid": "CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "date_updated": "Fri, 01 Jan 2021 11:23:45 +0000",
                        "date_created": "Fri, 01 Jan 2021 11:23:45 +0000",
                        "subresource_uris": {
                            "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Participants.json",
                            "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Recordings.json"
                        },
                        "friendly_name": "friendly_name",
                        "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa.json",
                        "api_version": "2010-04-01",
                        "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "reason_conference_ended": null,
                        "call_sid_ending_conference": null
                    },
                    {
                        "status": "in-progress",
                        "region": "de1",
                        "sid": "CFbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb",
                        "date_updated": "Fri, 01 Jan 2021 11:23:45 +0000",
                        "date_created": "Fri, 01 Jan 2021 11:23:45 +0000",
                        "subresource_uris": {
                            "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb/Participants.json",
                            "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb/Recordings.json"
                        },
                        "friendly_name": "MyRoom",
                        "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb.json",
                        "api_version": "2010-04-01",
                        "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "reason_conference_ended": null,
                        "call_sid_ending_conference": null
                    },
                    {
                        "status": "in-progress",
                        "region": "br1",
                        "sid": "CFcccccccccccccccccccccccccccccccc",
                        "date_updated": "Fri, 01 Jan 2021 11:23:45 +0000",
                        "date_created": "Fri, 01 Jan 2021 11:23:45 +0000",
                        "subresource_uris": {
                            "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFcccccccccccccccccccccccccccccccc/Participants.json",
                            "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFcccccccccccccccccccccccccccccccc/Recordings.json"
                        },
                        "friendly_name": "FRIEND",
                        "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFcccccccccccccccccccccccccccccccc.json",
                        "api_version": "2010-04-01",
                        "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "reason_conference_ended": null,
                        "call_sid_ending_conference": null
                    }
                ],
                "first_page_uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences.json?Status=in-progress&DateCreated%3E=2021-01-01&PageSize=20&Page=0",
                "next_page_uri": null,
                "previous_page_uri": null,
                "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences.json?Status=in-progress&DateCreated%3E=2021-01-01&PageSize=20&Page=0",
                "page": 0,
                "page_size": 20,
                "start": 0,
                "end": 2
            }
            '''
        ))

        actual = self.client.api.v2010.accounts("ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                      .conferences.list()

        self.assertIsNotNone(actual)

    def test_update_request(self):
        self.holodeck.mock(Response(500, ''))

        with self.assertRaises(TwilioException):
            self.client.api.v2010.accounts("ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                 .conferences("CFXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX").update()

        self.holodeck.assert_has_request(Request(
            'post',
            'https://api.twilio.com/2010-04-01/Accounts/ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX/Conferences/CFXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX.json',
        ))

    def test_update_end_conference_response(self):
        self.holodeck.mock(Response(
            200,
            '''
            {
                "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "api_version": "2010-04-01",
                "date_created": "Mon, 22 Aug 2011 20:58:45 +0000",
                "date_updated": "Mon, 22 Aug 2011 20:58:46 +0000",
                "friendly_name": null,
                "region": "us1",
                "sid": "CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "status": "completed",
                "subresource_uris": {
                    "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Participants.json",
                    "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Recordings.json"
                },
                "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa.json",
                "reason_conference_ended": "conference-ended-via-api",
                "call_sid_ending_conference": null
            }
            '''
        ))

        actual = self.client.api.v2010.accounts("ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                      .conferences("CFXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX").update()

        self.assertIsNotNone(actual)

    def test_announce_to_conference_response(self):
        self.holodeck.mock(Response(
            200,
            '''
            {
                "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "api_version": "2010-04-01",
                "date_created": "Mon, 08 Feb 2021 20:58:45 +0000",
                "date_updated": "Mon, 08 Feb 2021 20:58:46 +0000",
                "friendly_name": "MyRoom",
                "region": "us1",
                "sid": "CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "status": "in-progress",
                "subresource_uris": {
                    "participants": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Participants.json",
                    "recordings": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Recordings.json"
                },
                "uri": "/2010-04-01/Accounts/ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Conferences/CFaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa.json",
                "reason_conference_ended": null,
                "call_sid_ending_conference": null
            }
            '''
        ))

        actual = self.client.api.v2010.accounts("ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                      .conferences("CFXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX").update()

        self.assertIsNotNone(actual)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from tests import IntegrationTestCase
from tests.holodeck import Request
from twilio.base.exceptions import TwilioException
from twilio.http.response import Response


class BrandVettingTestCase(IntegrationTestCase):

    def test_create_request(self):
        self.holodeck.mock(Response(500, ''))

        with self.assertRaises(TwilioException):
            self.client.messaging.v1.brand_registrations("BNXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                    .brand_vettings.create(vetting_provider="campaign-verify")

        values = {'VettingProvider': "campaign-verify", }

        self.holodeck.assert_has_request(Request(
            'post',
            'https://messaging.twilio.com/v1/a2p/BrandRegistrations/BNXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX/Vettings',
            data=values,
        ))

    def test_create_response(self):
        self.holodeck.mock(Response(
            201,
            '''
            {
                "account_sid": "AC78e8e67fc0246521490fb9907fd0c165",
                "brand_sid": "BN0044409f7e067e279523808d267e2d85",
                "brand_vetting_sid": "VT12445353",
                "vetting_provider": "campaign-verify",
                "vetting_id": "cv|1.0|tcr|10dlc|9975c339-d46f-49b7-a399-EXAMPLETOKEN|GQ3EXAMPLETOKENAXXBUNBT2AgL-LdQuPveFhEyY",
                "vetting_class": "POLITICAL",
                "vetting_status": "IN_PROGRESS",
                "date_created": "2021-01-27T14:18:35Z",
                "date_updated": "2021-01-27T14:18:35Z",
                "url": "https://messaging.twilio.com/v1/a2p/BrandRegistrations/BN0044409f7e067e279523808d267e2d85/Vettings/VT12445353"
            }
            '''
        ))

        actual = self.client.messaging.v1.brand_registrations("BNXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                         .brand_vettings.create(vetting_provider="campaign-verify")

        self.assertIsNotNone(actual)

    def test_list_request(self):
        self.holodeck.mock(Response(500, ''))

        with self.assertRaises(TwilioException):
            self.client.messaging.v1.brand_registrations("BNXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                    .brand_vettings.list()

        self.holodeck.assert_has_request(Request(
            'get',
            'https://messaging.twilio.com/v1/a2p/BrandRegistrations/BNXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX/Vettings',
        ))

    def test_read_response(self):
        self.holodeck.mock(Response(
            200,
            '''
            {
                "meta": {
                    "page": 0,
                    "page_size": 50,
                    "first_page_url": "https://messaging.twilio.com/v1/a2p/BrandRegistrations/BN0044409f7e067e279523808d267e2d85/Vettings?PageSize=50&Page=0",
                    "previous_page_url": null,
                    "next_page_url": null,
                    "key": "data",
                    "url": "https://messaging.twilio.com/v1/a2p/BrandRegistrations/BN0044409f7e067e279523808d267e2d85/Vettings?PageSize=50&Page=0"
                },
                "data": [
                    {
                        "account_sid": "AC78e8e67fc0246521490fb9907fd0c165",
                        "brand_sid": "BN0044409f7e067e279523808d267e2d85",
                        "brand_vetting_sid": "VT12445353",
                        "vetting_provider": "campaign-verify",
                        "vetting_id": "cv|1.0|tcr|10dlc|9975c339-d46f-49b7-a399-EXAMPLETOKEN|GQ3EXAMPLETOKENAXXBUNBT2AgL-LdQuPveFhEyY",
                        "vetting_class": "POLITICAL",
                        "vetting_status": "IN_PROGRESS",
                        "date_created": "2021-01-27T14:18:35Z",
                        "date_updated": "2021-01-27T14:18:35Z",
                        "url": "https://messaging.twilio.com/v1/a2p/BrandRegistrations/BN0044409f7e067e279523808d267e2d85/Vettings/VT12445353"
                    }
                ]
            }
            '''
        ))

        actual = self.client.messaging.v1.brand_registrations("BNXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                         .brand_vettings.list()

        self.assertIsNotNone(actual)

    def test_fetch_request(self):
        self.holodeck.mock(Response(500, ''))

        with self.assertRaises(TwilioException):
            self.client.messaging.v1.brand_registrations("BNXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                    .brand_vettings("VTXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX").fetch()

        self.holodeck.assert_has_request(Request(
            'get',
            'https://messaging.twilio.com/v1/a2p/BrandRegistrations/BNXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX/Vettings/VTXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX',
        ))

    def test_fetch_response(self):
        self.holodeck.mock(Response(
            200,
            '''
            {
                "account_sid": "AC78e8e67fc0246521490fb9907fd0c165",
                "brand_sid": "BN0044409f7e067e279523808d267e2d85",
                "brand_vetting_sid": "VT12445353",
                "vetting_provider": "campaign-verify",
                "vetting_id": "cv|1.0|tcr|10dlc|9975c339-d46f-49b7-a399-EXAMPLETOKEN|GQ3EXAMPLETOKENAXXBUNBT2AgL-LdQuPveFhEyY",
                "vetting_class": "POLITICAL",
                "vetting_status": "IN_PROGRESS",
                "date_created": "2021-01-27T14:18:35Z",
                "date_updated": "2021-01-27T14:18:35Z",
                "url": "https://messaging.twilio.com/v1/a2p/BrandRegistrations/BN0044409f7e067e279523808d267e2d85/Vettings/VT12445353"
            }
            '''
        ))

        actual = self.client.messaging.v1.brand_registrations("BNXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                         .brand_vettings("VTXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX").fetch()

        self.assertIsNotNone(actual)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from tests import IntegrationTestCase
from tests.holodeck import Request
from twilio.base.exceptions import TwilioException
from twilio.http.response import Response


class TaskActionsTestCase(IntegrationTestCase):

    def test_fetch_request(self):
        self.holodeck.mock(Response(500, ''))

        with self.assertRaises(TwilioException):
            self.client.autopilot.v1.assistants("UAXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                    .tasks("UDXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                    .task_actions().fetch()

        self.holodeck.assert_has_request(Request(
            'get',
            'https://autopilot.twilio.com/v1/Assistants/UAXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX/Tasks/UDXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX/Actions',
        ))

    def test_fetch_response(self):
        self.holodeck.mock(Response(
            200,
            '''
            {
                "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "assistant_sid": "UAaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "task_sid": "UDdddddddddddddddddddddddddddddddd",
                "data": {},
                "url": "https://autopilot.twilio.com/v1/Assistants/UAaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Tasks/UDdddddddddddddddddddddddddddddddd/Actions"
            }
            '''
        ))

        actual = self.client.autopilot.v1.assistants("UAXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                         .tasks("UDXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                         .task_actions().fetch()

        self.assertIsNotNone(actual)

    def test_update_request(self):
        self.holodeck.mock(Response(500, ''))

        with self.assertRaises(TwilioException):
            self.client.autopilot.v1.assistants("UAXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                    .tasks("UDXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                    .task_actions().update()

        self.holodeck.assert_has_request(Request(
            'post',
            'https://autopilot.twilio.com/v1/Assistants/UAXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX/Tasks/UDXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX/Actions',
        ))

    def test_update_response(self):
        self.holodeck.mock(Response(
            200,
            '''
            {
                "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "assistant_sid": "UAaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "task_sid": "UDdddddddddddddddddddddddddddddddd",
                "data": {},
                "url": "https://autopilot.twilio.com/v1/Assistants/UAaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa/Tasks/UDdddddddddddddddddddddddddddddddd/Actions"
            }
            '''
        ))

        actual = self.client.autopilot.v1.assistants("UAXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                         .tasks("UDXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX") \
                                         .task_actions().update()

        self.assertIsNotNone(actual)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from tests import IntegrationTestCase
from tests.holodeck import Request
from twilio.base.exceptions import TwilioException
from twilio.http.response import Response


class CountryTestCase(IntegrationTestCase):

    def test_list_request(self):
        self.holodeck.mock(Response(500, ''))

        with self.assertRaises(TwilioException):
            self.client.pricing.v2.voice \
                                  .countries.list()

        self.holodeck.assert_has_request(Request(
            'get',
            'https://pricing.twilio.com/v2/Voice/Countries',
        ))

    def test_read_full_response(self):
        self.holodeck.mock(Response(
            200,
            '''
            {
                "countries": [
                    {
                        "country": "Andorra",
                        "iso_country": "AD",
                        "url": "https://pricing.twilio.com/v2/Voice/Countries/AD"
                    }
                ],
                "meta": {
                    "first_page_url": "https://pricing.twilio.com/v2/Voice/Countries?PageSize=50&Page=0",
                    "key": "countries",
                    "next_page_url": null,
                    "page": 0,
                    "page_size": 50,
                    "previous_page_url": null,
                    "url": "https://pricing.twilio.com/v2/Voice/Countries?PageSize=50&Page=0"
                }
            }
            '''
        ))

        actual = self.client.pricing.v2.voice \
                                       .countries.list()

        self.assertIsNotNone(actual)

    def test_read_empty_response(self):
        self.holodeck.mock(Response(
            200,
            '''
            {
                "countries": [],
                "meta": {
                    "first_page_url": "https://pricing.twilio.com/v2/Voice/Countries?PageSize=50&Page=0",
                    "key": "countries",
                    "next_page_url": null,
                    "page": 0,
                    "page_size": 50,
                    "previous_page_url": null,
                    "url": "https://pricing.twilio.com/v2/Voice/Countries?PageSize=50&Page=0"
                }
            }
            '''
        ))

        actual = self.client.pricing.v2.voice \
                                       .countries.list()

        self.assertIsNotNone(actual)

    def test_fetch_request(self):
        self.holodeck.mock(Response(500, ''))

        with self.assertRaises(TwilioException):
            self.client.pricing.v2.voice \
                                  .countries("US").fetch()

        self.holodeck.assert_has_request(Request(
            'get',
            'https://pricing.twilio.com/v2/Voice/Countries/US',
        ))

    def test_fetch_response(self):
        self.holodeck.mock(Response(
            200,
            '''
            {
                "country": "United States",
                "inbound_call_prices": [
                    {
                        "base_price": "0.0085",
                        "current_price": "0.0085",
                        "number_type": "local"
                    },
                    {
                        "base_price": "0.022",
                        "current_price": "0.022",
                        "number_type": "toll free"
                    }
                ],
                "iso_country": "US",
                "outbound_prefix_prices": [
                    {
                        "base_price": "0.090",
                        "current_price": "0.090",
                        "destination_prefixes": [
                            "1907"
                        ],
                        "friendly_name": "Programmable Outbound Minute - United States - Alaska",
                        "origination_prefixes": [
                            "ALL"
                        ]
                    },
                    {
                        "base_price": "0.013",
                        "current_price": "0.013",
                        "destination_prefixes": [
                            "1808"
                        ],
                        "friendly_name": "Programmable Outbound Minute - United States - Hawaii",
                        "origination_prefixes": [
                            "ALL"
                        ]
                    },
                    {
                        "base_price": "0.013",
                        "current_price": "0.013",
                        "destination_prefixes": [
                            "1800",
                            "1844",
                            "1855",
                            "1866",
                            "1877",
                            "1888"
                        ],
                        "friendly_name": "Programmable Outbound Minute - United States & Canada - Toll Free",
                        "origination_prefixes": [
                            "ALL"
                        ]
                    },
                    {
                        "base_price": "0.013",
                        "current_price": "0.013",
                        "destination_prefixes": [
                            "1"
                        ],
                        "friendly_name": "Programmable Outbound Minute - United States & Canada",
                        "origination_prefixes": [
                            "ALL"
                        ]
                    }
                ],
                "price_unit": "USD",
                "url": "https://pricing.twilio.com/v2/Voice/Countries/US"
            }
            '''
        ))

        actual = self.client.pricing.v2.voice \
                                       .countries("US").fetch()

        self.assertIsNotNone(actual)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import serialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class AddressConfigurationList(ListResource):

    def __init__(self, version):
        """
        Initialize the AddressConfigurationList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.conversations.v1.address_configuration.AddressConfigurationList
        :rtype: twilio.rest.conversations.v1.address_configuration.AddressConfigurationList
        """
        super(AddressConfigurationList, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/Configuration/Addresses'.format(**self._solution)

    def stream(self, limit=None, page_size=None):
        """
        Streams AddressConfigurationInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.conversations.v1.address_configuration.AddressConfigurationInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, limit=None, page_size=None):
        """
        Lists AddressConfigurationInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.conversations.v1.address_configuration.AddressConfigurationInstance]
        """
        return list(self.stream(limit=limit, page_size=page_size, ))

    def page(self, page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of AddressConfigurationInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of AddressConfigurationInstance
        :rtype: twilio.rest.conversations.v1.address_configuration.AddressConfigurationPage
        """
        data = values.of({'PageToken': page_token, 'Page': page_number, 'PageSize': page_size, })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return AddressConfigurationPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of AddressConfigurationInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of AddressConfigurationInstance
        :rtype: twilio.rest.conversations.v1.address_configuration.AddressConfigurationPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return AddressConfigurationPage(self._version, response, self._solution)

    def create(self, type, address, friendly_name=values.unset,
               auto_creation_enabled=values.unset, auto_creation_type=values.unset,
               auto_creation_conversation_service_sid=values.unset,
               auto_creation_webhook_url=values.unset,
               auto_creation_webhook_method=values.unset,
               auto_creation_webhook_filters=values.unset,
               auto_creation_studio_flow_sid=values.unset,
               auto_creation_studio_retry_count=values.unset):
        """
        Create the AddressConfigurationInstance

        :param AddressConfigurationInstance.Type type: Type of Address.
        :param unicode address: The unique address to be configured.
        :param unicode friendly_name: The human-readable name of this configuration.
        :param bool auto_creation_enabled: Enable/Disable auto-creating conversations for messages to this address
        :param AddressConfigurationInstance.AutoCreationType auto_creation_type: Type of Auto Creation.
        :param unicode auto_creation_conversation_service_sid: Conversation Service for the auto-created conversation.
        :param unicode auto_creation_webhook_url: For type `webhook`, the url for the webhook request.
        :param AddressConfigurationInstance.Method auto_creation_webhook_method: For type `webhook`, the HTTP method to be used when sending a webhook request.
        :param list[unicode] auto_creation_webhook_filters: The list of events, firing webhook event for this Conversation.
        :param unicode auto_creation_studio_flow_sid: For type `studio`, the studio flow SID where the webhook should be sent to.
        :param unicode auto_creation_studio_retry_count: For type `studio`, number of times to retry the webhook request

        :returns: The created AddressConfigurationInstance
        :rtype: twilio.rest.conversations.v1.address_configuration.AddressConfigurationInstance
        """
        data = values.of({
            'Type': type,
            'Address': address,
            'FriendlyName': friendly_name,
            'AutoCreation.Enabled': auto_creation_enabled,
            'AutoCreation.Type': auto_creation_type,
            'AutoCreation.ConversationServiceSid': auto_creation_conversation_service_sid,
            'AutoCreation.WebhookUrl': auto_creation_webhook_url,
            'AutoCreation.WebhookMethod': auto_creation_webhook_method,
            'AutoCreation.WebhookFilters': serialize.map(auto_creation_webhook_filters, lambda e: e),
            'AutoCreation.StudioFlowSid': auto_creation_studio_flow_sid,
            'AutoCreation.StudioRetryCount': auto_creation_studio_retry_count,
        })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return AddressConfigurationInstance(self._version, payload, )

    def get(self, sid):
        """
        Constructs a AddressConfigurationContext

        :param sid: The SID or Address of the Configuration.

        :returns: twilio.rest.conversations.v1.address_configuration.AddressConfigurationContext
        :rtype: twilio.rest.conversations.v1.address_configuration.AddressConfigurationContext
        """
        return AddressConfigurationContext(self._version, sid=sid, )

    def __call__(self, sid):
        """
        Constructs a AddressConfigurationContext

        :param sid: The SID or Address of the Configuration.

        :returns: twilio.rest.conversations.v1.address_configuration.AddressConfigurationContext
        :rtype: twilio.rest.conversations.v1.address_configuration.AddressConfigurationContext
        """
        return AddressConfigurationContext(self._version, sid=sid, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Conversations.V1.AddressConfigurationList>'


class AddressConfigurationPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the AddressConfigurationPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.conversations.v1.address_configuration.AddressConfigurationPage
        :rtype: twilio.rest.conversations.v1.address_configuration.AddressConfigurationPage
        """
        super(AddressConfigurationPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of AddressConfigurationInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.conversations.v1.address_configuration.AddressConfigurationInstance
        :rtype: twilio.rest.conversations.v1.address_configuration.AddressConfigurationInstance
        """
        return AddressConfigurationInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Conversations.V1.AddressConfigurationPage>'


class AddressConfigurationContext(InstanceContext):

    def __init__(self, version, sid):
        """
        Initialize the AddressConfigurationContext

        :param Version version: Version that contains the resource
        :param sid: The SID or Address of the Configuration.

        :returns: twilio.rest.conversations.v1.address_configuration.AddressConfigurationContext
        :rtype: twilio.rest.conversations.v1.address_configuration.AddressConfigurationContext
        """
        super(AddressConfigurationContext, self).__init__(version)

        # Path Solution
        self._solution = {'sid': sid, }
        self._uri = '/Configuration/Addresses/{sid}'.format(**self._solution)

    def fetch(self):
        """
        Fetch the AddressConfigurationInstance

        :returns: The fetched AddressConfigurationInstance
        :rtype: twilio.rest.conversations.v1.address_configuration.AddressConfigurationInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return AddressConfigurationInstance(self._version, payload, sid=self._solution['sid'], )

    def update(self, friendly_name=values.unset, auto_creation_enabled=values.unset,
               auto_creation_type=values.unset,
               auto_creation_conversation_service_sid=values.unset,
               auto_creation_webhook_url=values.unset,
               auto_creation_webhook_method=values.unset,
               auto_creation_webhook_filters=values.unset,
               auto_creation_studio_flow_sid=values.unset,
               auto_creation_studio_retry_count=values.unset):
        """
        Update the AddressConfigurationInstance

        :param unicode friendly_name: The human-readable name of this configuration.
        :param bool auto_creation_enabled: Enable/Disable auto-creating conversations for messages to this address
        :param AddressConfigurationInstance.AutoCreationType auto_creation_type: Type of Auto Creation.
        :param unicode auto_creation_conversation_service_sid: Conversation Service for the auto-created conversation.
        :param unicode auto_creation_webhook_url: For type `webhook`, the url for the webhook request.
        :param AddressConfigurationInstance.Method auto_creation_webhook_method: For type `webhook`, the HTTP method to be used when sending a webhook request.
        :param list[unicode] auto_creation_webhook_filters: The list of events, firing webhook event for this Conversation.
        :param unicode auto_creation_studio_flow_sid: For type `studio`, the studio flow SID where the webhook should be sent to.
        :param unicode auto_creation_studio_retry_count: For type `studio`, number of times to retry the webhook request

        :returns: The updated AddressConfigurationInstance
        :rtype: twilio.rest.conversations.v1.address_configuration.AddressConfigurationInstance
        """
        data = values.of({
            'FriendlyName': friendly_name,
            'AutoCreation.Enabled': auto_creation_enabled,
            'AutoCreation.Type': auto_creation_type,
            'AutoCreation.ConversationServiceSid': auto_creation_conversation_service_sid,
            'AutoCreation.WebhookUrl': auto_creation_webhook_url,
            'AutoCreation.WebhookMethod': auto_creation_webhook_method,
            'AutoCreation.WebhookFilters': serialize.map(auto_creation_webhook_filters, lambda e: e),
            'AutoCreation.StudioFlowSid': auto_creation_studio_flow_sid,
            'AutoCreation.StudioRetryCount': auto_creation_studio_retry_count,
        })

        payload = self._version.update(method='POST', uri=self._uri, data=data, )

        return AddressConfigurationInstance(self._version, payload, sid=self._solution['sid'], )

    def delete(self):
        """
        Deletes the AddressConfigurationInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(method='DELETE', uri=self._uri, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Conversations.V1.AddressConfigurationContext {}>'.format(context)


class AddressConfigurationInstance(InstanceResource):

    class Type(object):
        SMS = "sms"
        WHATSAPP = "whatsapp"
        MESSENGER = "messenger"
        GBM = "gbm"

    class AutoCreationType(object):
        WEBHOOK = "webhook"
        STUDIO = "studio"
        DEFAULT = "default"

    class Method(object):
        GET = "GET"
        POST = "POST"

    def __init__(self, version, payload, sid=None):
        """
        Initialize the AddressConfigurationInstance

        :returns: twilio.rest.conversations.v1.address_configuration.AddressConfigurationInstance
        :rtype: twilio.rest.conversations.v1.address_configuration.AddressConfigurationInstance
        """
        super(AddressConfigurationInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'account_sid': payload.get('account_sid'),
            'type': payload.get('type'),
            'address': payload.get('address'),
            'friendly_name': payload.get('friendly_name'),
            'auto_creation': payload.get('auto_creation'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {'sid': sid or self._properties['sid'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: AddressConfigurationContext for this AddressConfigurationInstance
        :rtype: twilio.rest.conversations.v1.address_configuration.AddressConfigurationContext
        """
        if self._context is None:
            self._context = AddressConfigurationContext(self._version, sid=self._solution['sid'], )
        return self._context

    @property
    def sid(self):
        """
        :returns: A 34 character string that uniquely identifies this resource.
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def account_sid(self):
        """
        :returns: The unique ID of the Account the address belongs to.
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def type(self):
        """
        :returns: Type of Address.
        :rtype: unicode
        """
        return self._properties['type']

    @property
    def address(self):
        """
        :returns: The unique address to be configured.
        :rtype: unicode
        """
        return self._properties['address']

    @property
    def friendly_name(self):
        """
        :returns: The human-readable name of this configuration.
        :rtype: unicode
        """
        return self._properties['friendly_name']

    @property
    def auto_creation(self):
        """
        :returns: Auto Creation configuration for the address.
        :rtype: dict
        """
        return self._properties['auto_creation']

    @property
    def date_created(self):
        """
        :returns: The date that this resource was created.
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The date that this resource was last updated.
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def url(self):
        """
        :returns: An absolute URL for this address configuration.
        :rtype: unicode
        """
        return self._properties['url']

    def fetch(self):
        """
        Fetch the AddressConfigurationInstance

        :returns: The fetched AddressConfigurationInstance
        :rtype: twilio.rest.conversations.v1.address_configuration.AddressConfigurationInstance
        """
        return self._proxy.fetch()

    def update(self, friendly_name=values.unset, auto_creation_enabled=values.unset,
               auto_creation_type=values.unset,
               auto_creation_conversation_service_sid=values.unset,
               auto_creation_webhook_url=values.unset,
               auto_creation_webhook_method=values.unset,
               auto_creation_webhook_filters=values.unset,
               auto_creation_studio_flow_sid=values.unset,
               auto_creation_studio_retry_count=values.unset):
        """
        Update the AddressConfigurationInstance

        :param unicode friendly_name: The human-readable name of this configuration.
        :param bool auto_creation_enabled: Enable/Disable auto-creating conversations for messages to this address
        :param AddressConfigurationInstance.AutoCreationType auto_creation_type: Type of Auto Creation.
        :param unicode auto_creation_conversation_service_sid: Conversation Service for the auto-created conversation.
        :param unicode auto_creation_webhook_url: For type `webhook`, the url for the webhook request.
        :param AddressConfigurationInstance.Method auto_creation_webhook_method: For type `webhook`, the HTTP method to be used when sending a webhook request.
        :param list[unicode] auto_creation_webhook_filters: The list of events, firing webhook event for this Conversation.
        :param unicode auto_creation_studio_flow_sid: For type `studio`, the studio flow SID where the webhook should be sent to.
        :param unicode auto_creation_studio_retry_count: For type `studio`, number of times to retry the webhook request

        :returns: The updated AddressConfigurationInstance
        :rtype: twilio.rest.conversations.v1.address_configuration.AddressConfigurationInstance
        """
        return self._proxy.update(
            friendly_name=friendly_name,
            auto_creation_enabled=auto_creation_enabled,
            auto_creation_type=auto_creation_type,
            auto_creation_conversation_service_sid=auto_creation_conversation_service_sid,
            auto_creation_webhook_url=auto_creation_webhook_url,
            auto_creation_webhook_method=auto_creation_webhook_method,
            auto_creation_webhook_filters=auto_creation_webhook_filters,
            auto_creation_studio_flow_sid=auto_creation_studio_flow_sid,
            auto_creation_studio_retry_count=auto_creation_studio_retry_count,
        )

    def delete(self):
        """
        Deletes the AddressConfigurationInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Conversations.V1.AddressConfigurationInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from tests import IntegrationTestCase
from tests.holodeck import Request
from twilio.base.exceptions import TwilioException
from twilio.http.response import Response


class ConfigurationTestCase(IntegrationTestCase):

    def test_fetch_request(self):
        self.holodeck.mock(Response(500, ''))

        with self.assertRaises(TwilioException):
            self.client.flex_api.v1.configuration().fetch()

        self.holodeck.assert_has_request(Request(
            'get',
            'https://flex-api.twilio.com/v1/Configuration',
        ))

    def test_fetch_response(self):
        self.holodeck.mock(Response(
            200,
            '''
            {
                "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "date_created": "2016-08-01T22:10:40Z",
                "date_updated": "2016-08-01T22:10:40Z",
                "attributes": {
                    "main_attribute": "some_attribute"
                },
                "status": "ok",
                "taskrouter_workspace_sid": "WSaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "taskrouter_target_workflow_sid": "WWaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "taskrouter_target_taskqueue_sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "taskrouter_taskqueues": [
                    {
                        "sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaab",
                        "targettable": true
                    },
                    {
                        "sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaac",
                        "targettable": false
                    }
                ],
                "taskrouter_skills": [
                    {
                        "name": "sales",
                        "multivalue": false,
                        "minimum": 0,
                        "maximum": 0
                    },
                    {
                        "name": "support",
                        "multivalue": true,
                        "minimum": 0,
                        "maximum": 10
                    }
                ],
                "taskrouter_worker_channels": {
                    "agent": [
                        {
                            "name": "default",
                            "availability": true,
                            "capacity": 1
                        },
                        {
                            "name": "voice",
                            "availability": false,
                            "capacity": 2
                        }
                    ],
                    "supervisor": [
                        {
                            "name": "default",
                            "availability": true,
                            "capacity": 2
                        }
                    ]
                },
                "taskrouter_worker_attributes": {
                    "agent": {
                        "region": "us-east"
                    },
                    "supervisor": {
                        "region": "us"
                    }
                },
                "taskrouter_offline_activity_sid": "WAaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "runtime_domain": "https://flex.twilio.com",
                "messaging_service_instance_sid": "MGaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "chat_service_instance_sid": "ISaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "flex_service_instance_sid": "ISaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "ui_language": "en",
                "ui_attributes": {},
                "ui_dependencies": {},
                "ui_version": "1.0",
                "service_version": "1.0",
                "call_recording_enabled": true,
                "call_recording_webhook_url": "https://www.example.com/call-recording",
                "crm_enabled": true,
                "crm_type": "custom",
                "crm_callback_url": "https://crm.com/a",
                "crm_fallback_url": "https://crm.com/b",
                "crm_attributes": {
                    "crm_attribute": "some_crm"
                },
                "public_attributes": {
                    "public": "test"
                },
                "plugin_service_enabled": true,
                "plugin_service_attributes": {
                    "agent-logger": "^3.10.5",
                    "typewriter": "^7.0.1"
                },
                "integrations": [
                    {
                        "name": "twilio",
                        "type": "http",
                        "active": true,
                        "config": "{\\"callback\\":\\"twilio.com/cb\\",\\"allowed_methods\\":[\\"GET\\",\\"POST\\"]}",
                        "logo": "logo1",
                        "author": "somebody1"
                    },
                    {
                        "name": "twilio-stage",
                        "type": "http",
                        "active": false,
                        "config": "{\\"callback\\":\\"twilio.com/cb\\",\\"allowed_methods\\":[\\"GET\\",\\"POST\\"]}"
                    }
                ],
                "outbound_call_flows": {
                    "default": {
                        "caller_id": "+12345",
                        "queue_sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "location": "EE",
                        "workflow_sid": "WWaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
                    }
                },
                "queue_stats_configuration": {
                    "default": {
                        "service_level_threshold": 20,
                        "short_abandoned_threshold": 5,
                        "reset_timezone": "America/New_York",
                        "reset_time": "00:00"
                    },
                    "queue_configurations": [
                        {
                            "queue_sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                            "reset_timezone": "Europe/Tallinn",
                            "reset_time": "01:00"
                        },
                        {
                            "queue_sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaab",
                            "reset_timezone": "Europe/Paris",
                            "reset_time": "02:00"
                        }
                    ],
                    "queue_channel_configurations": [
                        {
                            "queue_sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                            "channel_sid": "TCaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                            "service_level_threshold": 10,
                            "short_abandoned_threshold": 10
                        },
                        {
                            "queue_sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaab",
                            "channel_sid": "TCaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaab",
                            "service_level_threshold": 30,
                            "short_abandoned_threshold": 15
                        }
                    ]
                },
                "serverless_service_sids": [
                    "ZSaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                    "ZSaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaab"
                ],
                "notifications": {
                    "enabled": true,
                    "mode": "whenNotInFocus"
                },
                "markdown": {
                    "enabled": false,
                    "mode": "readOnly"
                },
                "flex_insights_hr": {
                    "advanced_reporting_enabled": true,
                    "advanced_reporting_workspaces": [
                        {
                            "id": "ID_1",
                            "timezone": "America/Los_Angeles",
                            "workspace_id": "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
                        },
                        {
                            "id": "ID_2",
                            "timezone": "Etc/UTC"
                        }
                    ]
                },
                "url": "https://flex-api.twilio.com/v1/Configuration",
                "flex_insights_drilldown": true,
                "flex_url": "https://flex.twilio.com"
            }
            '''
        ))

        actual = self.client.flex_api.v1.configuration().fetch()

        self.assertIsNotNone(actual)

    def test_create_request(self):
        self.holodeck.mock(Response(500, ''))

        with self.assertRaises(TwilioException):
            self.client.flex_api.v1.configuration().create()

        self.holodeck.assert_has_request(Request(
            'post',
            'https://flex-api.twilio.com/v1/Configuration',
        ))

    def test_create_response(self):
        self.holodeck.mock(Response(
            201,
            '''
            {
                "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "date_created": "2016-08-01T22:10:40Z",
                "date_updated": "2016-08-01T22:10:40Z",
                "attributes": {
                    "main_attribute": "some_attribute"
                },
                "status": "ok",
                "taskrouter_workspace_sid": "WSaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "taskrouter_target_workflow_sid": "WWaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "taskrouter_target_taskqueue_sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "taskrouter_taskqueues": [
                    {
                        "sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaab",
                        "targettable": true
                    },
                    {
                        "sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaac",
                        "targettable": false
                    }
                ],
                "taskrouter_skills": [
                    {
                        "name": "sales",
                        "multivalue": false,
                        "minimum": 0,
                        "maximum": 0
                    },
                    {
                        "name": "support",
                        "multivalue": true,
                        "minimum": 0,
                        "maximum": 10
                    }
                ],
                "taskrouter_worker_channels": {
                    "agent": [
                        {
                            "name": "default",
                            "availability": true,
                            "capacity": 1
                        },
                        {
                            "name": "voice",
                            "availability": false,
                            "capacity": 2
                        }
                    ],
                    "supervisor": [
                        {
                            "name": "default",
                            "availability": true,
                            "capacity": 2
                        }
                    ]
                },
                "taskrouter_worker_attributes": {
                    "agent": {
                        "region": "us-east"
                    },
                    "supervisor": {
                        "region": "us"
                    }
                },
                "taskrouter_offline_activity_sid": "WAaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "runtime_domain": "https://flex.twilio.com",
                "messaging_service_instance_sid": "MGaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "chat_service_instance_sid": "ISaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "flex_service_instance_sid": "ISaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "ui_language": "en",
                "ui_attributes": {},
                "ui_dependencies": {},
                "ui_version": "1.0",
                "service_version": "1.0",
                "call_recording_enabled": true,
                "call_recording_webhook_url": "https://www.example.com/call-recording",
                "crm_enabled": true,
                "crm_type": "custom",
                "crm_callback_url": "https://crm.com/a",
                "crm_fallback_url": "https://crm.com/b",
                "crm_attributes": {
                    "crm_attribute": "some_crm"
                },
                "public_attributes": {
                    "public": "test"
                },
                "plugin_service_enabled": true,
                "plugin_service_attributes": {
                    "agent-logger": "^3.10.5",
                    "typewriter": "^7.0.1"
                },
                "integrations": [
                    {
                        "name": "twilio",
                        "type": "http",
                        "active": true,
                        "config": "{\\"callback\\":\\"twilio.com/cb\\",\\"allowed_methods\\":[\\"GET\\",\\"POST\\"]}",
                        "logo": "logo1",
                        "author": "somebody1"
                    },
                    {
                        "name": "twilio-stage",
                        "type": "http",
                        "active": false,
                        "config": "{\\"callback\\":\\"twilio.com/cb\\",\\"allowed_methods\\":[\\"GET\\",\\"POST\\"]}"
                    }
                ],
                "outbound_call_flows": {
                    "default": {
                        "caller_id": "+12345",
                        "queue_sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "location": "EE",
                        "workflow_sid": "WWaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
                    }
                },
                "queue_stats_configuration": {
                    "default": {
                        "service_level_threshold": 20,
                        "short_abandoned_threshold": 5,
                        "reset_timezone": "America/New_York",
                        "reset_time": "00:00"
                    },
                    "queue_configurations": [
                        {
                            "queue_sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                            "reset_timezone": "Europe/Tallinn",
                            "reset_time": "01:00"
                        },
                        {
                            "queue_sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaab",
                            "reset_timezone": "Europe/Paris",
                            "reset_time": "02:00"
                        }
                    ],
                    "queue_channel_configurations": [
                        {
                            "queue_sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                            "channel_sid": "TCaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                            "service_level_threshold": 10,
                            "short_abandoned_threshold": 10
                        },
                        {
                            "queue_sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaab",
                            "channel_sid": "TCaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaab",
                            "service_level_threshold": 30,
                            "short_abandoned_threshold": 15
                        }
                    ]
                },
                "serverless_service_sids": [
                    "ZSaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                    "ZSaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaab"
                ],
                "notifications": {
                    "enabled": true,
                    "mode": "whenNotInFocus"
                },
                "markdown": {
                    "enabled": false,
                    "mode": "readOnly"
                },
                "flex_insights_hr": null,
                "url": "https://flex-api.twilio.com/v1/Configuration",
                "flex_insights_drilldown": false,
                "flex_url": "https://flex.twilio.com"
            }
            '''
        ))

        actual = self.client.flex_api.v1.configuration().create()

        self.assertIsNotNone(actual)

    def test_update_request(self):
        self.holodeck.mock(Response(500, ''))

        with self.assertRaises(TwilioException):
            self.client.flex_api.v1.configuration().update()

        self.holodeck.assert_has_request(Request(
            'post',
            'https://flex-api.twilio.com/v1/Configuration',
        ))

    def test_update_response(self):
        self.holodeck.mock(Response(
            200,
            '''
            {
                "account_sid": "ACaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "date_created": "2016-08-01T22:10:40Z",
                "date_updated": "2016-08-01T22:10:40Z",
                "attributes": {
                    "main_attribute": "some_attribute"
                },
                "status": "ok",
                "taskrouter_workspace_sid": "WSaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "taskrouter_target_workflow_sid": "WWaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "taskrouter_target_taskqueue_sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "taskrouter_taskqueues": [
                    {
                        "sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaab",
                        "targettable": true
                    },
                    {
                        "sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaac",
                        "targettable": false
                    }
                ],
                "taskrouter_skills": [
                    {
                        "name": "sales",
                        "multivalue": false,
                        "minimum": 0,
                        "maximum": 0
                    },
                    {
                        "name": "support",
                        "multivalue": true,
                        "minimum": 0,
                        "maximum": 10
                    }
                ],
                "taskrouter_worker_channels": {
                    "agent": [
                        {
                            "name": "default",
                            "availability": true,
                            "capacity": 1
                        },
                        {
                            "name": "voice",
                            "availability": false,
                            "capacity": 2
                        }
                    ],
                    "supervisor": [
                        {
                            "name": "default",
                            "availability": true,
                            "capacity": 2
                        }
                    ]
                },
                "taskrouter_worker_attributes": {
                    "agent": {
                        "region": "us-east"
                    },
                    "supervisor": {
                        "region": "us"
                    }
                },
                "taskrouter_offline_activity_sid": "WAaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "runtime_domain": "https://flex.twilio.com",
                "messaging_service_instance_sid": "MGaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "chat_service_instance_sid": "ISaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "flex_service_instance_sid": "ISaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                "ui_language": "en",
                "ui_attributes": {},
                "ui_dependencies": {},
                "ui_version": "1.0",
                "service_version": "1.0",
                "call_recording_enabled": true,
                "call_recording_webhook_url": "https://www.example.com/call-recording",
                "crm_enabled": true,
                "crm_type": "custom",
                "crm_callback_url": "https://crm.com/a",
                "crm_fallback_url": "https://crm.com/b",
                "crm_attributes": {
                    "crm_attribute": "some_crm"
                },
                "public_attributes": {
                    "public": "test"
                },
                "plugin_service_enabled": false,
                "plugin_service_attributes": {
                    "agent-logger": "^3.10.5",
                    "typewriter": "^7.0.1"
                },
                "integrations": [
                    {
                        "name": "twilio",
                        "type": "http",
                        "active": true,
                        "config": "{\\"callback\\":\\"twilio.com/cb\\",\\"allowed_methods\\":[\\"GET\\",\\"POST\\"]}",
                        "logo": "logo1",
                        "author": "somebody1"
                    },
                    {
                        "name": "twilio-stage",
                        "type": "http",
                        "active": false,
                        "config": "{\\"callback\\":\\"twilio.com/cb\\",\\"allowed_methods\\":[\\"GET\\",\\"POST\\"]}"
                    }
                ],
                "outbound_call_flows": {
                    "default": {
                        "caller_id": "+12345",
                        "queue_sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "location": "EE",
                        "workflow_sid": "WWaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
                    }
                },
                "queue_stats_configuration": {
                    "default": {
                        "service_level_threshold": 20,
                        "short_abandoned_threshold": 5,
                        "reset_timezone": "America/New_York",
                        "reset_time": "00:00"
                    },
                    "queue_configurations": [
                        {
                            "queue_sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                            "reset_timezone": "Europe/Tallinn",
                            "reset_time": "01:00"
                        },
                        {
                            "queue_sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaab",
                            "reset_timezone": "Europe/Paris",
                            "reset_time": "02:00"
                        }
                    ],
                    "queue_channel_configurations": [
                        {
                            "queue_sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                            "channel_sid": "TCaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                            "service_level_threshold": 10,
                            "short_abandoned_threshold": 10
                        },
                        {
                            "queue_sid": "WQaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaab",
                            "channel_sid": "TCaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaab",
                            "service_level_threshold": 30,
                            "short_abandoned_threshold": 15
                        }
                    ]
                },
                "serverless_service_sids": [
                    "ZSaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                    "ZSaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaab"
                ],
                "notifications": {
                    "enabled": true,
                    "mode": "whenNotInFocus"
                },
                "markdown": {
                    "enabled": false,
                    "mode": "readOnly"
                },
                "flex_insights_hr": {
                    "advanced_reporting_enabled": true,
                    "advanced_reporting_workspaces": [
                        {
                            "id": "ID_1",
                            "timezone": "America/Los_Angeles",
                            "workspace_id": "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
                        },
                        {
                            "id": "ID_2",
                            "timezone": "Etc/UTC"
                        }
                    ]
                },
                "url": "https://flex-api.twilio.com/v1/Configuration",
                "flex_insights_drilldown": true,
                "flex_url": "https://flex.twilio.com"
            }
            '''
        ))

        actual = self.client.flex_api.v1.configuration().update()

        self.assertIsNotNone(actual)
